#ifndef __ARITHVEC_H
#define __ARITHVEC_H

#include <assert.h>
#include <iostream>

template <class T, class V, int N> 
class ArithVec
{
  T data[N];

public:

	typedef T ScalarType;
	typedef V VectorType;
	static int get_dim() {return N;}

#define for_all_i(expr) for(int i=0;i<N;i++) {expr;}
	
  //----------------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------------

  ArithVec() 
	{
		for_all_i(data[i]=0)
	}
  explicit ArithVec(T _a)
	{
		for_all_i(data[i] = _a)
	}
	ArithVec(T _a, T _b)
	{
		assert(N==2);
		data[0] = _a;
		data[1] = _b;
	}
  ArithVec(T _a, T _b, T _c)
	{
		assert(N==3);
		data[0] = _a;
		data[1] = _b;
		data[2] = _c;
	}
  ArithVec(T _a, T _b, T _c, T _d)
	{
		assert(N==4);
		data[0] = _a;
		data[1] = _b;
		data[2] = _c;
		data[3] = _d;
	}
	
	void set(T _a, T _b)
	{
		assert(N==2);
		data[0] = _a;
		data[1] = _b;
	}

  void set(T _a, T _b, T _c)
	{
		assert(N==3);
		data[0] = _a;
		data[1] = _b;
		data[2] = _c;
	}

  void set(T _a, T _b, T _c, T _d)
	{
		assert(N==4);
		data[0] = _a;
		data[1] = _b;
		data[2] = _c;
		data[3] = _d;
	}

  //----------------------------------------------------------------------
  // index operators
  //----------------------------------------------------------------------

	const T& operator [] ( int i ) const
	{
		assert(i<N);
		return data[i];
	}

	T& operator [] ( int i ) 
	{
		assert(i<N);
		return data[i];
	}

  //----------------------------------------------------------------------
  // Comparison operators
  //----------------------------------------------------------------------

  bool operator==(const V& v) const 
	{
		for_all_i(if (data[i] != v[i]) return false)
		return true;
	}

  bool operator==(T k) const 
	{ 
		for_all_i(if (data[i] != k) return false)
		return true;
	}


  //----------------------------------------------------------------------
  // Comparison functions ... of geometric significance 
  //----------------------------------------------------------------------


  bool  all_l  (const V& v) const
	{
		for_all_i(if (data[i] >= v[i]) return false)
		return true;
	}

  bool  all_le (const V& v) const
	{
		for_all_i(if (data[i] > v[i]) return false)
		return true;
	}

  bool  all_g  (const V& v) const
	{
		for_all_i(if (data[i] <= v[i]) return false)
		return true;
	}

  bool  all_ge (const V& v) const
	{
		for_all_i(if (data[i] < v[i]) return false);
		return true;
	}


  //----------------------------------------------------------------------
  // Assignment operators
  //----------------------------------------------------------------------

  void operator *=(T k) { for_all_i(data[i] *= k); }

  void operator /=(T k) { for_all_i(data[i] /= k); }

  void operator +=(T k) { for_all_i(data[i] += k); }

  void operator -=(T k) { for_all_i(data[i] -= k); }

  void operator *=(const V& v) { for_all_i(data[i] *= v[i]); }

  void operator /=(const V& v) { for_all_i(data[i] /= v[i]); }

  void operator +=(const V& v) { for_all_i(data[i] += v[i]); }

  void operator -=(const V& v) { for_all_i(data[i] -= v[i]); }


  //----------------------------------------------------------------------
  // Unary operators on vectors
  //----------------------------------------------------------------------

  const V operator - () const
	{
		V v_new;
		for_all_i(v_new[i] = - data[i]);
		return v_new;
	}

  //----------------------------------------------------------------------
  // Binary operators on vectors
  //----------------------------------------------------------------------

  const V operator * (const V& v1) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] * v1[i]);
		return v_new;
	}

  const V operator + (const V& v1) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] + v1[i]);
		return v_new;
	}

  const V operator - (const V& v1) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] - v1[i]);
		return v_new;
	}

  const V operator / (const V& v1) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] / v1[i]);
		return v_new;
	}

  //----------------------------------------------------------------------
  // Binary operators on vector and scalar
  //----------------------------------------------------------------------

  const V operator * (T k) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] * k);
		return v_new;
	}
  

  const V operator / (T k) const
	{
		V v_new;
		for_all_i(v_new[i] = data[i] / k);
		return v_new;      
	}

#undef for_all_i  
};

#endif

