#ifndef BITMASK_H
#define BITMASK_H

#include "Vec3i.h"

const int MASKS[32] = 
{
  0x00000001,
  0x00000003,
  0x00000007,
  0x0000000f,
  0x0000001f,
  0x0000003f,
  0x0000007f,
  0x000000ff,
  0x000001ff,
  0x000003ff,
  0x000007ff,
  0x00000fff,
  0x00001fff,
  0x00003fff,
  0x00007fff,
  0x0000ffff,
  0x0001ffff,
  0x0003ffff,
  0x0007ffff,
  0x000fffff,
  0x001fffff,
  0x003fffff,
  0x007fffff,
  0x00ffffff,
  0x01ffffff,
  0x03ffffff,
  0x07ffffff,
  0x0fffffff,
  0x1fffffff,
  0x3fffffff,
  0x7fffffff,
  0xffffffff
};

const int powers_of_2[] = 
{
	1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768
};

class BitMask
{
  int msk;
  int fb, lb, bdiff;
public:

  BitMask(int _fb, int _lb):fb(_fb), lb(_lb) 
    {
      bdiff=lb-fb;
      msk = MASKS[bdiff]<<fb;
    }

  BitMask():fb(0), lb(15) {}

	int length() const {return lb-fb+1;}

  int mask(int var) const {return msk&var;}
  int maskshift(int var) const {return (msk&var)>>fb;}

	// An alternative, perhaps more elegant/portable version is sketched 
	// below. If the stuff below runs 
	// 
	//   int mask(int var) const 
	// 	{
	// 		return ((var%powers_of_2[lb])/powers_of_2[fb])*powers_of_2[fb];
	// 	}
	//   int maskshift(int var) const 
	// 	{
	// 		return (var%powers_of_2[lb])/powers_of_2[fb];
	// 	}

  Vec3i mask(const Vec3i& v) const 
	{
		return Vec3i(mask(v[0]),mask(v[1]),mask(v[2]));
	}
  
	Vec3i maskshift(const Vec3i& v) const 
	{
		return Vec3i(maskshift(v[0]),maskshift(v[1]),maskshift(v[2]));
	}
  
};

#endif
