#ifndef __QUATERNION_H
#define __QUATERNION_H

#include <math.h>
#include "CGLA/Vec3f.h"
#include "CGLA/Vec4f.h"
#include "CGLA/Mat4x4f.h"

#ifdef WIN32
#define M_PI 3.1415926
#endif

class Quaternion
{
public:
	Vec3f qv;
	float qw;

	Quaternion::Quaternion() {}

	Quaternion(const Vec3f _qv, float _qw) : 
		qv(_qv) , qw(_qw) {}

	Quaternion(float x, float y, float z, float _qw) : 
		qv(x,y,z), qw(_qw) {}

	void set(float x, float y, float z, float _qw) 
	{
		qv.set(x,y,z);
		qw = _qw;
	}

	void get(float x, float y, float z, float _qw) const
	{
		x  = qv[0];
		y  = qv[1];
		z  = qv[2];
		_qw = qw;
	}

	Mat4x4f Quaternion::get_mat() const;
	void make_rot(float angle, float x, float y, float z);

	Quaternion operator *(Quaternion quat) const;
	Quaternion operator *(float scalar) const;
	Quaternion operator +(Quaternion quat) const;

	Quaternion inverse();
	Quaternion conjugate();

	float norm() const;
	Quaternion normalized();
	Vec3f apply(Vec3f vec);
};

inline std::ostream& operator<<(std::ostream&os, const Quaternion v)
{
  os << "[ ";
  for(int i=0;i<3;i++) os << v.qv[i] << " ";
	os << v.qw << " ";
	os << "]";

  return os;
}

inline Quaternion Quaternion::operator *(Quaternion quat) const
{
	return Quaternion(cross(qv,quat.qv) + quat.qw*qv + qw*quat.qv, 
										qw*quat.qw - dot(qv,quat.qv));
}

inline Quaternion Quaternion::operator *(float scalar) const
{
	return Quaternion(scalar*qv,scalar*qw);
}

inline Quaternion operator *(float scalar, Quaternion quat) 
{
	return Quaternion(scalar*quat.qv,scalar*quat.qw);
}

inline Quaternion Quaternion::operator +(Quaternion quat) const
{
	return Quaternion(qv + quat.qv,qw + quat.qw);
}

inline Quaternion Quaternion::inverse() 
{
	return Quaternion(1/norm()*conjugate());
}

inline Quaternion Quaternion::conjugate() 
{
	return Quaternion(-qv,qw);
}

inline Quaternion Quaternion::normalized() 
{
	return Quaternion(1/norm()*(*this));
}

inline float Quaternion::norm() const
{
	return qv[0]*qv[0] + qv[1]*qv[1] + qv[2]*qv[2] + qw*qw;
}



#endif //__QUATERNION_H
