#ifndef VECDAT_H
#define VECDAT_H

template<class T, int N> 
struct VecDat { };

//----------------------------------------------------------------------
// VecDat<T,2>
//----------------------------------------------------------------------

template<class T> struct VecDat<T,2>
{
  T a,b;

  VecDat() {} 
  VecDat(T x):a(x),b(x) {}
  VecDat(T _a, T _b):a(_a),b(_b) {}
  VecDat(T _a, T _b, T _c) {assert(0);}
  VecDat(T _a, T _b, T _c, T _d) {assert(0);}


  template<class AOP> 
  void apply_assign(const VecDat& v) 
    {
      AOP::apply(a,v.a);
      AOP::apply(b,v.b);
    }

  template<class BOP> 
  void apply(const VecDat& v0, const VecDat& v1) 
    {
      a = BOP::apply(v0.a,v1.a);
      b = BOP::apply(v0.b,v1.b);
    }
  
  template<class ROP> 
  bool compare_all(const VecDat& v) const
    {
      return ( ROP::apply( a, v.a ) &&
               ROP::apply( b, v.b ) );
	      
    }

  const T& operator [] ( int i ) const
    {
      assert(i<2);
      switch( i ) 
        {
        case 0: return a;
        }
      return b;
    }

  T& operator [] ( int i )
    {
      assert(i<2);
      switch( i ) 
        {
        case 0: return a;
        }
      return b;
    }

  bool operator <(const VecDat& x) const
    {
      return a==x.a?(b==x.b?false:b<x.b):a<x.a;
    }
};


//----------------------------------------------------------------------
// VecDat<T,3>
//----------------------------------------------------------------------


template<class T> struct VecDat<T,3>
{
  T a,b,c;

  VecDat() {}
  VecDat(T x):a(x), b(x), c(x) {}  
  VecDat(T _a, T _b) {assert(0);}
  VecDat(T _a, T _b, T _c):a(_a),b(_b),c(_c) {}
  VecDat(T _a, T _b, T _c, T _d) {assert(0);}

  template<class AOP> 
  void apply_assign(const VecDat& v) 
    {
      AOP::apply(a,v.a);
      AOP::apply(b,v.b);
      AOP::apply(c,v.c);
    }

  template<class BOP> 
  void apply(const VecDat& v0, const VecDat& v1) 
    {
      a = BOP::apply(v0.a,v1.a);
      b = BOP::apply(v0.b,v1.b);
      c = BOP::apply(v0.c,v1.c);
    }
  
  template<class ROP> 
  bool compare_all(const VecDat& v) const
    {
      return ( ROP::apply( a, v.a ) &&
               ROP::apply( b, v.b ) &&
               ROP::apply( c, v.c ) );
	      
    }

  const T& operator [] ( int i ) const
    {
      assert(i<3);
      switch( i ) 
        {
        case 0: return a;
        case 1: return b;
        }
      return c;
    }

  T& operator [] ( int i )
    {
      assert(i<3);
      switch( i ) 
        {
        case 0: return a;
        case 1: return b;
        }
      return c;
    }

  bool operator <(const VecDat& x) const
    {
      return a==x.a?(b==x.b?(c==x.c?false:c<x.c):b<x.b):a<x.a;
    }
};





//----------------------------------------------------------------------
// VecDat<T,4>
//----------------------------------------------------------------------



template<class T> struct VecDat<T,4>
{
  T a,b,c,d;

  VecDat() {}
  VecDat(T x):a(x),b(x),c(x),d(x) {}
  VecDat(T _a, T _b) {assert(0);}
  VecDat(T _a, T _b, T _c) {assert(0);}
  VecDat(T _a, T _b, T _c, T _d):a(_a),b(_b),c(_c),d(_d) {}


  template<class AOP> 
  void apply_assign(const VecDat& v) 
    {
      AOP::apply(a,v.a);
      AOP::apply(b,v.b);
      AOP::apply(c,v.c);
      AOP::apply(d,v.d);
    }

  template<class BOP> 
  void apply(const VecDat& v0, const VecDat& v1) 
    {
      a = BOP::apply(v0.a,v1.a);
      b = BOP::apply(v0.b,v1.b);
      c = BOP::apply(v0.c,v1.c);
      c = BOP::apply(v0.d,v1.d);
    }
  
  template<class ROP> 
  bool compare_all(const VecDat& v) const
    {
      return ( ROP::apply( a, v.a ) &&
               ROP::apply( b, v.b ) &&
               ROP::apply( c, v.c ) &&
               ROP::apply( d, v.d ) );
    }

  const T& operator [] ( int i ) const
    {
      assert(i<4);
      switch( i ) 
        {
        case 0: return a;
        case 1: return b;
        case 2: return c;
        }
      return d;
    }

  T& operator [] ( int i )
    {
      assert(i<4);
      switch( i ) 
        {
        case 0: return a;
        case 1: return b;
        case 2: return c;
        }
      return d;
    }

  bool operator <(const VecDat& x) const
    {
      return a==x.a?(b==x.b?(c==x.c?(d==x.d?false:d<x.d):c<x.c):b<x.b):a<x.a;
    }
};


#endif








