// Author: J. Andreas Brentzen,
// Created: Thu Mar 30 12:49:0

#include <assert.h>
//#include <iostream>
#include "StateKeeper.h"

using namespace std;

ActionHistory hist;

void StateKeeper::undo_last_action()
{
	// If we are not on the first action, we move back in the history
	if(hist.go_back())
		{
			// and pop the state - this must succeed.
			bool state_popped = pop_state();
			assert(state_popped);
			// Reset aux variables
			reset_from_state();
		}
}


void StateKeeper::redo_last_undone_action()
{
	// If we are not at the end of history
	if(!hist.at_end())
		{
			// We push state and redo action
			push_state();
			hist.redo();

			// Reset aux variables
			reset_from_state();

			// And perform post action stuff.
			bool post_action_succeeded = post_action();

			// Post action must succeed since this is a redo, and it 
			// has worked before.
			assert(post_action_succeeded);
		}
}

bool StateKeeper::try_action(Action* action)
{
	// Push state and add action to history
	push_state();
	hist.add(action);

	// Now apply the action
	if(hist.apply_last() && post_action()) 
		return true;
	else
		{
			// Otherwise, we remove the action, pop state, rebuild
			// the other variables and report an error
			hist.un_add();
			pop_state();
			reset_from_state();
		}

	return false;
}
