#include <time.h>
#include <GL/gl.h>
#include <iostream>

#include "Common/CommonDefs.h"
#include "AccelRenderEngine.h"
#include "Graphics/PPMBitmap.h"

using namespace std;

void AccelRenderEngine::render()
{
  // init random number generator
  srandom(clock());
  double rand_max = pow(2.0,31) - 1.0;
  float mindepth=10, maxdepth=-10;
  for (int x=0;x<canvas->get_width();x++)
    {
      for(int y=0;y<canvas->get_height(); y++)
        {
          float depth;
          if (do_accel)
            depth = zmap->depthpixel(x,y);

          if(!do_accel || depth < zmap->get_far())
            {
              ColorVector c;
              for (int i=0;i<samplesx;i++)
                for (int j=0;j<samplesy;j++)
                  {
                    float ipx, ipy;
                    float randnum1, randnum2;
                    
                    if(do_jitter)
                      {
                        randnum1 = double(random())/rand_max;
                        randnum2 = double(random())/rand_max;
                      }
                    else
                      {
                        randnum1 = 0.5;
                        randnum2 = 0.5;
                      }

                    // Calculate the jittered pixel positions.
                    double jpx =
                      double(x) 
                      + (double(i) + randnum1)
                      /  double(samplesx);
                    
                    double jpy = 
                      double(y) 
                      + (double(j) + randnum2)
                      /  double(samplesy);
                    
                    canvas->window_coords2image_coords(jpx,jpy,ipx,ipy);
                
                    Ray r(eye->eye_position,
                          eye->get_ray_direction(ipx,ipy), SMALL);
                    
                    if(do_accel) r.set_parameter(depth);
                    c += shader->shade_ray(r) * weight;
                    canvas->draw_pixel(x,y,c);
                  }
            }
        }
      cout << "." << flush ;
    }
  cout << "min" << mindepth << endl;
  cout << "max" << maxdepth << endl;

}
