
#ifndef __ACCELRENDERENGINE_H
#define __ACCELRENDERENGINE_H

#include "Eye.h"
#include "Shader.h"
#include "RenderCanvas.h"
#include "RenderEngine.h"
#include "DepthMap.h"

///
class AccelRenderEngine: public RenderEngine
{
  Shader* shader;
  RenderCanvas* canvas;
  Eye* eye;
  DepthMap* zmap;
  
  /// Number of samples in x direction and y direction
  int samplesx, samplesy;

  /// Should we accelerate.
  bool do_accel;

  /// Should we shake it ?
  bool do_jitter;

  /// The weight of each samples in the case of multiple samples per pixel.
  double weight;
  
public:

  ///
  AccelRenderEngine(Shader*       _shader, 
                    RenderCanvas* _canvas,
                    Eye*          _eye,
                    DepthMap*     _zmap,
                    int _samplesx, int _samplesy,
                    bool _do_accel,
                    bool _do_jitter = false ): 
    shader(_shader), canvas(_canvas), eye(_eye), 
    zmap(_zmap), samplesx(_samplesx), samplesy(_samplesy),
    do_accel(_do_accel),
    do_jitter(_do_jitter)
    {
      weight = 1.0 / float(samplesx * samplesy);
    }
  
  ///
  void render();
};

#endif
