#include <assert.h>
#include <fstream>

#include "PPMBitmap.h"

const char PPMBitmap::prefixstr[] = "P6\n# GK incorporated \n%d %d\n255\n";

using namespace std;

PPMBitmap::PPMBitmap(const int x, const int y):
  xsz(x), ysz(y), bitmap_file(0), pixel_counter(0)
{
  bitmap = new unsigned char[4*xsz*ysz];
  if(!bitmap)
    {
      cerr << "could not allocate bitmap" << endl;
      exit(1);
    }
}

PPMBitmap::PPMBitmap(const char* filename)
  :bitmap_file(0), pixel_counter(0)
{
  // 0x81 means read only | binary file ... make symbolic
	fstream f(filename, /*0x81*/ ios::in | ios::binary );
	
	if (!f)
    {
      cerr << "Couldn't open file"  << filename << endl; 
      exit(1);
    }
  else
    {
      char c;
      while(f.read(&c,1) && c!= 0x0a);
			

      // Read comments

      while(f.read(&c,1) && c == '#')
        while(f.read(&c,1) && c!= 0x0a);
      
      // Read size
      char sizestr[80] = {c,'\0'};
      int i = 1;
      while(f.read(sizestr + i,1) && sizestr[i++] != 0x0a);
      // Scan x,y size
      sscanf(sizestr,"%d %d", &xsz, &ysz);

      if (xsz == 0 || ysz == 0)
        {
          cerr << "Bad image file header" << endl;
          exit(1);
        }

      while(f.read(&c,1) && c!= 0x0a);  // read till newline

      // Allocate bitmap 
      bitmap = new unsigned char[4*xsz*ysz];
			
			assert(bitmap);	
			
			//read the pixels and put them in bitmap
      for(int y= 0; y<ysz; y++)
        for(int x= 0; x<xsz; x++)
          {
            char pixels[4]= {0,0,0,0};
					  f.read(pixels, 3);
						memcpy(pix(x,ysz-y-1), pixels, 4);
          }
    }
}

ColorVector PPMBitmap::get_pixel(const int x, const int y) const
{
  double red, green, blue;

  red   = double(*(pix(x,y)))  / 256.0;
  green = double(*(pix(x,y)+1)) / 256.0;
  blue  = double(*(pix(x,y)+2)) / 256.0;

  return ColorVector(red, green ,blue);
}

  
bool PPMBitmap::writebitmap(const char* filename) const
{
  ofstream f(filename);
  if(f)
    {
      char str[80];
			char *bitmap_ptr = (char*)(bitmap);
      sprintf(str, prefixstr, xsz, ysz);
      f.write(str, strlen(str));
      for(int i=0; i< xsz*ysz; i++) f.write(&(bitmap_ptr[i*4]), 3);
      return true;
    }
  return false;
}

bool PPMBitmap::open_file(const char* filename)
{
  bitmap_file = new ofstream(filename);
  if (*bitmap_file)
    {
      char str[80];
      sprintf(str, prefixstr, xsz, ysz);
      bitmap_file->write( str, strlen(str));
      bitmap_file->flush();
      return true;
    }
  else return false;
}


void PPMBitmap::set_pixel_file(const ColorVector& cv)
{
  int x = pixel_counter % xsz;
  int y = pixel_counter / xsz;
  cv.set_PPM_style(pix(x,y));
  bitmap_file->write((char*)(&bitmap[pixel_counter*4]), 3);
  pixel_counter++;
}

	
bool PPMBitmap::close_file()
{
  delete bitmap_file;
  pixel_counter = 0;
  return 1;
}






