#ifndef __PPMBITMAP_H
#define __PPMBITMAP_H

#ifdef __sgi
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <math.h>
#include <fstream>
#include <iostream>
#include "Graphics/ColorVector.h"

 /** An OpenGL compatible bitmap that can be read and written
    as a ppm file */
class PPMBitmap
{
  /// The prefix is in the header when we write the image to disk
  static const char prefixstr[];
  
  /// a pointer to the bitmap
  unsigned char * bitmap;

  /// Size of the image
  int xsz, ysz;

  /** Pointer to a file to which the image is being written. Used in cases
      where we start writing the image before all pixels are available */
  std::ofstream* bitmap_file;

  /** Used to keep track of how much of the image has been computed */
  long pixel_counter;

  /// Returns a pointer to pixel x, y
  unsigned char* pix(const int x, const int y)
    {
      return &(bitmap[4 * (xsz*y + x)]);
    }

  /// Returns a const pointer to pixel x,y
  const unsigned char* pix(const int x, const int y) const 
    {
      return &(bitmap[4 * (xsz*y + x)]);
    }

public:

//   /// Construct an empty bitmap... gratuitous ?!
//   PPMBitmap(): xsz(0), ysz(0), bitmap(0) {}

  /// Construct bitmap of specified size
  PPMBitmap(const int x, const int y);

  /// Construct and load bitmap from specified file.
  PPMBitmap(const char* filename);

  
  /// Destruct Bitmap
  ~PPMBitmap()
    {
      delete bitmap_file;
      delete [] bitmap;
    }

  /// Retrieve width of image
  int  get_x() const {return xsz;}

  /// Retrieve height of image
  int  get_y() const {return ysz;}

  /// Retrieve a pixel as a colour vector.
  ColorVector get_pixel(const int x, const int y) const;

  /// Get a pointer to the bitmap
  void* get_bitmap() {return bitmap;}

  /// Write the bitmap to disk in one go.
  bool writebitmap(const char* filename) const;
  
  /// Assign a colour to a pixel.
  void set_pixel(const int x, const int y, const ColorVector& cv)
    {
      cv.set_PPM_style(pix(x,y));
    }

  /// Open an image file for piecemeal output of bitmap.
  bool open_file(const char* filename);	

  // Write one pixel to bitmap and simultaneously to open file.
  void set_pixel_file(const ColorVector& cv);

  /// Close file used for piecemeal output of bitmap.
  bool close_file();

};


#endif
