#ifndef _TrackBall_
#define _TrackBall_

#include "CGLA/Vec3f.h"
#include "CGLA/Quaternion.h"

enum TrackBallAction
{
	NO_ACTION = 0,
  ROTATE_ACTION,
  PAN_ACTION,
  ZOOM_ACTION
};

/** This class represents a virtual tracball. 
		Use it in GLUT, FLTK or other OpenGL programs to allow the user to 
		spin the model being rendered. It needs work to be used with non-GL
		apps since it calls GL API functions. */
class TrackBall 
{
	Vec3f centre;

	unsigned width, height;
  Quaternion	qrot;
  Quaternion	qinc;
  Vec3f	trans;
  Vec3f scale;
  float	ballsize;
	float eye_dist;
  float	lastx, lasty;
	TrackBallAction current_action;

  void rotate(float, float);
  void pan(float, float);
  void zoom(float, float);

  void calcRotation(float, float, float, float);
  float projectToSphere(float, float);
  float scalePoint(long, long, long);

  void set_position(float, float);


public:

	/** First constructor argument is the point we look at. 
			The second argument is the distance to eye point.
			the last two arguments are the window dimensions. */
  TrackBall(const Vec3f&, float, unsigned, unsigned);

	/// Set window dimensions.
	void set_screen_window(unsigned _width, unsigned _height)
	{
		width = _width;
		height = _height;
	}
	
	/// set the centre point of rotation
	void set_centre(const Vec3f& _centre)
	{
		centre = _centre;
	}
	/// Call to reset the trackball and bring everything home to square one.
  void reset();

	/// Call GL to set up viewing. 
  void set_gl_modelview() const;

	/** Spin. used both to spin while button is pressed and if the ball is just
			spinning while program is idling. */
  void do_spin();
	
	/// Zeroes the rotation value - makes everything stop.
  void stop_spin();
	
	/// Call this function to start action when mouse button is pressed 
  void grab_ball(TrackBallAction,int,int);

	/// Call this function to perform action when user drags mouse
  void roll_ball(int,int);

	/// Call this function to stop action when mouse is released.
  void release_ball() 
	{
		current_action = NO_ACTION;
	}

	/// Returns true if the ball is `grabbed' and not released yet.
	bool is_grabbed() const 
	{
		if(current_action == NO_ACTION) 
			return false;
		return true;
	}

};

#endif
