#include <list>
#include <algobase.h>
#include <iostream>

#include <math.h>

#include "Common/CommonDefs.h"
#include "CGLA/Vec3Hf.h"
#include "CGLA/Mat4x4f.h"
#include "Math/Transformation.h"
#include "Math/AffineMatrixTransformation.h"

DERIVEEXCEPTION(NoSuchTransformation, MathException);


AffineMatrixTransformation::AffineMatrixTransformation(const Vec3f& v1, 
                                                       const Vec3f& v2, 
                                                       const Vec3f& v3)
{
  
  Mat4x4f m(Vec3Hf(v1,0),
            Vec3Hf(v2,0),
            Vec3Hf(v3,0),
            Vec3Hf(0,0,0,1)); 
  init_mtm(m);
  imtm = invert(m);
  init_hist(TransformationDesc(BASISTRANS,0,0,0));
}

AffineMatrixTransformation::AffineMatrixTransformation(CGLA::Axis axis,
						       float ang)
{
  switch(axis)
    {
    case CGLA::XAXIS:
      imtm = rotation_matrix(CGLA::XAXIS, -ang);
      init_mtm(rotation_matrix(CGLA::XAXIS, ang));
      init_hist(TransformationDesc(ROTATE, ang,0,0));
      return;
    case CGLA::YAXIS:
      imtm = rotation_matrix(CGLA::YAXIS, -ang);
      init_mtm(rotation_matrix(CGLA::YAXIS, ang));
      init_hist(TransformationDesc(ROTATE,0,ang,0));
      return;
    case CGLA::ZAXIS:
      imtm = rotation_matrix(CGLA::ZAXIS, -ang);
      init_mtm(rotation_matrix(CGLA::ZAXIS, ang));
      init_hist(TransformationDesc(ROTATE,0,0,ang));
      return;
    }
}

AffineMatrixTransformation::AffineMatrixTransformation(c_trans tf, 
																											 float x, 
																											 float y, 
																											 float z) 
{
  init_hist(TransformationDesc(tf,x,y,z));
  switch(tf)
    {
    case ROTATE:
      if (x != 0.0)
	{
	  imtm = rotation_matrix(CGLA::XAXIS, -x);
	  init_mtm(rotation_matrix(CGLA::XAXIS, x));
	}
      if (y != 0.0)
	{
	  imtm = rotation_matrix(CGLA::YAXIS, -y);
	  init_mtm(rotation_matrix(CGLA::YAXIS, y));
	}
      if (z != 0.0)
	{
	  imtm = rotation_matrix(CGLA::ZAXIS, -z);
	  init_mtm(rotation_matrix(CGLA::ZAXIS, z));
	}
      break;
    case TRANSLATE:
      imtm = translation_matrix(Vec3f(-x,-y,-z));
      init_mtm(translation_matrix(Vec3f(x,y,z)));
      break;
    case SCALE:
      imtm = scaling_matrix(Vec3f(1.0/x, 1.0/y, 1.0/z));
      init_mtm(scaling_matrix(Vec3f(x, y, z)));
      break;
    default:
      throw(NoSuchTransformation("Transformation not available"));
    }

}










