#ifndef __AFFINEMATRIXTRANSFORMATION_H
#define __AFFINEMATRIXTRANSFORMATION_H

#include "Common/CommonDefs.h"
#include "MatrixTransformation.h"


class AffineMatrixTransformation: public MatrixTransformation
{
  ///
  friend inline std::ostream& operator << (std::ostream& os, 
                                           const AffineMatrixTransformation& amt);
  
  ///
  Mat4x4f imtm;  
  
protected:
  
  ///
  AffineMatrixTransformation(const AffineMatrixTransformation& mt1,
			     const AffineMatrixTransformation& mt2):
    MatrixTransformation(mt1,mt2)
    { imtm = mt1.imtm * mt2.imtm; };
  
public:

  ///
  AffineMatrixTransformation(const  AffineMatrixTransformation& mt):
    MatrixTransformation(mt), imtm(mt.imtm) {};
  

  ///
  AffineMatrixTransformation() {imtm  = identity_matrix();}
  
  ///
  AffineMatrixTransformation(c_trans tf, 
                             float x, 
                             float y, 
                             float z);
  
  ///
  AffineMatrixTransformation(CGLA::Axis axis,
			     float ang);

  ///
  AffineMatrixTransformation(const Vec3f& v1, 
                             const Vec3f& v2, 
                             const Vec3f& v3);  
  
  ///
  Vec3Hf i(const Vec3Hf& v) const {return imtm * v;}

  
  AffineMatrixTransformation operator & (const AffineMatrixTransformation& t) 
    const
    {
      return AffineMatrixTransformation(*this,t);
    }

  ///
  void concat(const AffineMatrixTransformation& amt)
    {
      MatrixTransformation::concat(amt);
      imtm = imtm * amt.imtm;
    }
  
  ///
  void operator &= (const AffineMatrixTransformation& amt)
    {
      concat(amt);
    }

  ///
  AffineMatrixTransformation operator = (const AffineMatrixTransformation& amt)
    {
      return AffineMatrixTransformation(amt);
    }
  
  ///
  void print_trans(std::ostream& os) const
  {
    MatrixTransformation::print_trans(os);
    os << "\nInverse transformation matrix\n\n" << imtm;
  }
};


inline std::ostream& operator << (std::ostream& os, 
                                  const AffineMatrixTransformation& amt)
{
  amt.print_trans(os);
  return os;
}

#endif
