#ifndef __AXISPLANE_H
#define __AXISPLANE_H

#include "AbstractPlane.h"

/** An axisplane is a plane whose normal is parallel to one of \
    coordinate system axes. */
class AxisPlane: public AbstractPlane
{
  /// Which axis is perpendicular to the plane
	CGLA::Axis axis;

  /// In what direction does the normal point
  int sign;
  
protected:
  
  /// Calculate dot prodcut fast, since plane is axis aligned.
  virtual float n_dot(const Vec3f& v) const
    {
      return  (sign == 1) ? v[axis] : -v[axis];
    }

public:
  /// default constructor 
  AxisPlane(){};

  /** Create a plane by specifying a point, a perpendicular axis and a 
      direction */
  AxisPlane(const Vec3f& a, CGLA::Axis ax, int s): axis(ax), sign(s)
    {p = a; d = -n_dot(p);};

  /// Get the normal
  Vec3f normal(const Vec3f& dummy) const 
    {
      Vec3f n;
      n[axis] = 1.0;
      return n;
    }


};

#endif
