#ifndef __GEOMETRICALPRIMITIVE_H
#define __GEOMETRICALPRIMITIVE_H

#include "Common/CommonDefs.h"
#include "CGLA/Vec3f.h"
#include "Ray.h"

/** This abstract class represents the notion of a geometrical object.
    All functions that such objects should have in common are put here.
    */
class GeometricalPrimitive
{
public:
  /// Shortest distance to closest surface
  virtual float distance(const Vec3f&) const = 0;

  /// Finds first intersection with ray
  virtual bool intersection(Ray& r) const = 0;
  
  /// The normal at a point - not necessarily defined everywhere
  virtual Vec3f normal(const Vec3f&) const = 0;

  /// Intersection with box
  virtual bool box_intersect(const Vec3f&, const Vec3f&) const =0;

  /// Curvature at a point -  not necessarily defined everywhere
  virtual float curvature(const Vec3f&) const = 0;

	/** Args set to bounding box, if primitive is unbounded. Returns true 
			if the primitive is bounded false if unbounded. */
	virtual bool bounding_box(Vec3f&,Vec3f&) const {return false;}
};

#endif
