#ifndef __SPHERE_H
#define __SPHERE_H

#include "HesseNormalForm.h"
#include "CGLA/Vec3f.h"
#include "AbstractPlane.h"

///
class Sphere: public HesseNormalForm
{
  /// Centre of the sphere
  Vec3f centre;

  /// Radius of the sphere.
  float radius;

public:

  Sphere(const Vec3f& _centre, const float _radius): 
    centre(_centre), radius(_radius) {}
  
  bool plane_intersect(const AbstractPlane& plane, float& d) const;

  /// Finds first intersection with ray
  bool intersection(Ray& r) const;

  float distance(const Vec3f& pos) const;
  
  Vec3f normal(const Vec3f& pos) const;

  bool box_intersect(const Vec3f&,const Vec3f&) const;

  float curvature(const Vec3f& dummy) const {return 1.0 / radius;}

	float foot_point(const Vec3f&, Vec3f&) const;

	bool bounding_box(Vec3f& c0, Vec3f& c7) const 
	{
		Vec3f v = Vec3f(radius, radius, radius);
		c0 = centre - v;
		c7 = centre + v;
		return true;
	}
};

inline bool Sphere::plane_intersect(const AbstractPlane& plane, float& d) const
{
  d = plane.distance(centre);
  
  if (fabs(d) < radius)
    return true;
  return false;
}




inline float Sphere::distance(const Vec3f& pos) const
{
  Vec3f tmp = (pos-centre);
  return tmp.length() - radius;
}

inline Vec3f Sphere::normal(const Vec3f& pos) const
{
  Vec3f tmp = pos-centre;
  tmp.normalize();
  return tmp;
}


#endif
